class MCPHeroException(Exception):
    """Base MCPHero Exception class"""


INSTALL_GOOGLE_GENAI = """
To use Google Gemini with mcphero, please install dependencies:

pip install "mcphero[google-genai]"
"""
